;*********************************************************************
; File Name:	 ch4_e4.asm
; Target System: C240x Evaluation Board
; Description:	 This sample program measures the frequency of an input
;		 		square wave. Freq=4*pulses captured in 0.25 sec.
;
; Date:			12/30/2003
; Author:		Min Dai
;*********************************************************************

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Global symbol declarations
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        .def _c_int0,PHANTOM,GISR1,GISR2,GISR3,GISR4,GISR5,GISR6

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Address definitions
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        .include f2407.h

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Uninitialized global variable definitions
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        .bss    GPR0,1      ;general purpose variable
		.bss	CNT,1		;Counter for the rising edges
		.bss	FREQ,1		;Measurement result

;=====================================================================
; M A I N   C O D E  - starts here
;=====================================================================
		.text  
_c_int0:		
		NOP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Configure the System Control and Status Registers
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_PF1         ;set data page

        SPLK    #0000000011111101b, SCSR1
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15        0:      reserved
* bit 14        0:      CLKOUT = CPUCLK
* bit 13-12     00:     IDLE1 selected for low-power mode
* bit 11-9      000:    PLL x4 mode
* bit 8         0:      reserved
* bit 7         1:      1 = enable ADC module clock
* bit 6         1:      1 = enable SCI module clock
* bit 5         1:      1 = enable SPI module clock
* bit 4         1:      1 = enable CAN module clock
* bit 3         1:      1 = enable EVB module clock
* bit 2         1:      1 = enable EVA module clock
* bit 1         0:      reserved
* bit 0         1:      clear the ILLADR bit

        LACC    SCSR2                   ;ACC = SCSR2 register
        OR      #0000000000001011b      ;OR in bits to be set
        AND     #0000000000001111b      ;AND out bits to be cleared
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15-6      0's:    reserved
* bit 5         0:      do NOT clear the WD OVERRIDE bit
* bit 4         0:      XMIF_HI-Z, 0=normal mode, 1=Hi-Z'd
* bit 3         1:      disable the boot ROM, enable the FLASH
* bit 2     no change   MP/MC* bit reflects the state of the MP/MC* pin
* bit 1-0      11:      11 = SARAM mapped to prog and data (default)

        SACL    SCSR2                   ;store to SCSR2 register

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Other setup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup the core interrupts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #0h                     ;set data page
        SPLK    #0h,IMR                 ;clear the IMR register
        SPLK    #111111b,IFR            ;clear any pending core interrupts
        SPLK    #001010b,IMR            ;enable INT2, INT4 interrupts

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup the event manager interrupts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_EVA                 ;set data page
        SPLK    #0FFFFh, EVAIFRA        ;clear all EVA group A interrupts
        SPLK    #0FFFFh, EVAIFRB        ;clear all EVA group B interrupts
        SPLK    #0FFFFh, EVAIFRC        ;clear all EVA group C interrupts
        SPLK    #00000h, EVAIMRA        ;enabled desired EVA group A interrupts
        SPLK    #00001h, EVAIMRB        ;enabled desired EVA group B interrupts
        SPLK    #00000h, EVAIMRC        ;enabled desired EVA group C interrupts

        LDP     #DP_EVB                 ;set data page
        SPLK    #0FFFFh, EVBIFRA        ;clear all EVB group A interrupts
        SPLK    #0FFFFh, EVBIFRB        ;clear all EVB group B interrupts
        SPLK    #0FFFFh, EVBIFRC        ;clear all EVB group C interrupts
        SPLK    #00000h, EVBIMRA        ;enabled desired EVB group A interrupts
        SPLK    #00000h, EVBIMRB        ;enabled desired EVB group B interrupts
        SPLK    #00000h, EVBIMRC        ;enabled desired EVB group C interrupts

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Enable global interrupts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        CLRC    INTM                    ;enable global interrupts



;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Disable the watchdog timer
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_PF1                 ;set data page

        SPLK    #0000000011101000b, WDCR
*                ||||||||||||||||
*                FEDCBA9876543210
* bits 15-8     0's      reserved
* bit 7         1:       clear WD flag
* bit 6         1:       disable the dog
* bit 5-3       101:     must be written as 101
* bit 2-0       000:     WDCLK divider = 1

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup external memory interface for LF2407 EVM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #GPR0                  ;set current data page to 
        								;the data page of variable GPR0

        SPLK    #0000000001000000b, GPR0
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15-11     0's:    reserved
* bit 10-9      00:     bus visibility off
* bit 8-6       001:    1 wait-state for I/O space
* bit 5-3       000:    0 wait-state for data space
* bit 2-0       000:    0 wait state for program space

        OUT     GPR0, WSGR


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup shared I/O pins
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_PF2                 ;set data page

        SPLK    #0011000000111000b,MCRA ;set TxPWM pins
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15        0:      0=IOPB7,     1=TCLKINA
* bit 14        0:      0=IOPB6,     1=TDIRA
* bit 13        1:      0=IOPB5,     1=T2PWM/T2CMP
* bit 12        1:      0=IOPB4,     1=T1PWM/T1CMP
* bit 11        0:      0=IOPB3,     1=PWM6
* bit 10        0:      0=IOPB2,     1=PWM5
* bit 9         0:      0=IOPB1,     1=PWM4
* bit 8         0:      0=IOPB0,     1=PWM3
* bit 7         0:      0=IOPA7,     1=PWM2
* bit 6         0:      0=IOPA6,     1=PWM1
* bit 5         1:      0=IOPA5,     1=CAP3
* bit 4         1:      0=IOPA4,     1=CAP2/QEP2
* bit 3         1:      0=IOPA3,     1=CAP1/QEP1
* bit 2         0:      0=IOPA2,     1=XINT1
* bit 1         0:      0=IOPA1,     1=SCIRXD
* bit 0         0:      0=IOPA0,     1=SCITXD

;-------------------------------------------------------------------------
;		RESET SECTION - BEGINS
;-------------------------------------------------------------------------
        LDP		#DP_EVA                 ;set data page
		SPLK   	#0, GPTCONA
		SPLK   	#0, T1CON
		SPLK   	#0, T2CON

		SPLK   	#0, COMCONA
		SPLK   	#0, ACTRA
		SPLK   	#0, DBTCONA
		SPLK   	#0, CAPCONA

;-------------------------------------------------------------------------
;		RESET SECTION - ENDS
;-------------------------------------------------------------------------

        LDP		#DP_EVA         ;set data page
		SPLK   	#58594, T1PR	;Period=0.25 second (E4E2h)
								;T1PR=30000000/128/4 for 30MHz DSP
		SPLK   	#0045h, GPTCONA	;Enable compare outputs of all
								;GPTs. GPT1 compare output "Active
								;High"
		SPLK   	#0A040h, CAPCONA;Enable capture units 1 and 2
								;Select GP Timer 2 as time base for capture unit 1
								;This time base is not used in this program
								;Detect rising edge
		SPLK   	#0, CAPFIFOA	;Clear the capture unit FIFO initially
		SPLK   	#0080h, EVAIMRA	;Enable timer1 period interrupt
		SPLK   	#1, EVAIMRC		;Enable capture unit 1 interrupt

		LDP    	#CNT
		SPLK   	#0, CNT			;Initialize the edge counter

		LDP    	#DP_EVA

		SPLK   	#1746h, T1CON	;ENABLE GPT1
								;Input clock prescalar 1/128
								;Enable timer operations

		CLRC   	INTM			;Enable maskable interrupts

WAIT	B      WAIT

;=====================================================================
; ISR - GPT1_ISR
; Description:	Store the number of rising edges of input signal
;		every 1 second. Resets the counter which counts 
;		the number of rising edges.
; Modifies: FREQ, CNT
;=====================================================================
GISR2	
		LDP		#CNT        			;Set data page
		LT		CNT						;Load rising edge number into Temp Reg 
		MPY		#4                      ;Times 4 for 1 second
		SPL		FREQ					;Store the product as frequency
		SPLK	#0, CNT					;Reset rising edge counter
		LDP		#DP_EVA					;Set data page   
        SPLK    #0FFFFh, EVAIFRA        ;clear all EVA group A interrupts
		CLRC	INTM					;Enable maskable interrupts
		RET								;Return from interruption

;=====================================================================
; ISR - CAP1_ISR
; Description:	Counts the number of rising edges of input signal
;
; Modifies: FREQ, CNT
;=====================================================================
GISR4	
		LDP		#CNT
		LACC	CNT
		ADD		#1
		SACL	CNT					;Increment the edge counter by 1
		LDP		#DP_EVA
        SPLK    #0FFFFh, EVAIFRC	;clear all EVA group C interrupts
		CLRC	INTM				;Enable maskable interrupts
		RET							;Return from interruption
		

;=====================================================================
; I S R  -  PHANTOM
;
; Description:	Dummy ISR, used to trap spurious interrupts.
;
; Modifies:	Nothing
;=====================================================================
PHANTOM 	B	PHANTOM
GISR1		RET
;GISR2		RET
GISR3		RET
;GISR4		RET
GISR5		RET
GISR6		RET
