;*********************************************************************
; File Name:	 ch4_e5.asm
; Target System: C240x Evaluation Board
; Description:	 This counts the number of seconds elapsed and displays
;		 		the number on the LEDs on 2407 EVM
;
; Date:			01/01/2004
; Author:		Min Dai
;*********************************************************************

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Global symbol declarations
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        .def _c_int0,PHANTOM,GISR1,GISR2,GISR3,GISR4,GISR5,GISR6

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Address definitions
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        .include f2407.h

;---------------------------------------------------------------------
; I/O Mapped EVM Register Declarations
;---------------------------------------------------------------------
LEDS    .set    000Ch		;LEDs Register

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Uninitialized global variable definitions
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        .bss    GPR0,1      ;general purpose variable
		.bss	MSEC_CTR,1	;Milli-second counter
		.bss	SEC_CTR,1	;Second counter 

;=====================================================================
; M A I N   C O D E  - starts here
;=====================================================================
		.text  
_c_int0:		
		NOP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Configure the System Control and Status Registers
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_PF1         ;set data page

        SPLK    #0000000011111101b, SCSR1
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15        0:      reserved
* bit 14        0:      CLKOUT = CPUCLK
* bit 13-12     00:     IDLE1 selected for low-power mode
* bit 11-9      000:    PLL x4 mode
* bit 8         0:      reserved
* bit 7         1:      1 = enable ADC module clock
* bit 6         1:      1 = enable SCI module clock
* bit 5         1:      1 = enable SPI module clock
* bit 4         1:      1 = enable CAN module clock
* bit 3         1:      1 = enable EVB module clock
* bit 2         1:      1 = enable EVA module clock
* bit 1         0:      reserved
* bit 0         1:      clear the ILLADR bit

        LACC    SCSR2                   ;ACC = SCSR2 register
        OR      #0000000000001011b      ;OR in bits to be set
        AND     #0000000000001111b      ;AND out bits to be cleared
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15-6      0's:    reserved
* bit 5         0:      do NOT clear the WD OVERRIDE bit
* bit 4         0:      XMIF_HI-Z, 0=normal mode, 1=Hi-Z'd
* bit 3         1:      disable the boot ROM, enable the FLASH
* bit 2     no change   MP/MC* bit reflects the state of the MP/MC* pin
* bit 1-0      11:      11 = SARAM mapped to prog and data (default)

        SACL    SCSR2                   ;store to SCSR2 register

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Other setup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup the core interrupts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #0h                     ;set data page
        SPLK    #0h,IMR                 ;clear the IMR register
        SPLK    #111111b,IFR            ;clear any pending core interrupts
        SPLK    #000010b,IMR            ;enable INT2 interrupts

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup the event manager interrupts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_EVA                 ;set data page
        SPLK    #0FFFFh, EVAIFRA        ;clear all EVA group A interrupts
        SPLK    #0FFFFh, EVAIFRB        ;clear all EVA group B interrupts
        SPLK    #0FFFFh, EVAIFRC        ;clear all EVA group C interrupts
        SPLK    #00000h, EVAIMRA        ;enabled desired EVA group A interrupts
        SPLK    #00000h, EVAIMRB        ;enabled desired EVA group B interrupts
        SPLK    #00000h, EVAIMRC        ;enabled desired EVA group C interrupts

        LDP     #DP_EVB                 ;set data page
        SPLK    #0FFFFh, EVBIFRA        ;clear all EVB group A interrupts
        SPLK    #0FFFFh, EVBIFRB        ;clear all EVB group B interrupts
        SPLK    #0FFFFh, EVBIFRC        ;clear all EVB group C interrupts
        SPLK    #00000h, EVBIMRA        ;enabled desired EVB group A interrupts
        SPLK    #00000h, EVBIMRB        ;enabled desired EVB group B interrupts
        SPLK    #00000h, EVBIMRC        ;enabled desired EVB group C interrupts

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Enable global interrupts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        CLRC    INTM                    ;enable global interrupts



;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Disable the watchdog timer
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_PF1                 ;set data page

        SPLK    #0000000011101000b, WDCR
*                ||||||||||||||||
*                FEDCBA9876543210
* bits 15-8     0's      reserved
* bit 7         1:       clear WD flag
* bit 6         1:       disable the dog
* bit 5-3       101:     must be written as 101
* bit 2-0       000:     WDCLK divider = 1

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup external memory interface for LF2407 EVM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #GPR0                  ;set current data page to 
        								;the data page of variable GPR0

        SPLK    #0000000001000000b, GPR0
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15-11     0's:    reserved
* bit 10-9      00:     bus visibility off
* bit 8-6       001:    1 wait-state for I/O space
* bit 5-3       000:    0 wait-state for data space
* bit 2-0       000:    0 wait state for program space

        OUT     GPR0, WSGR

;-------------------------------------------------------------------------
;		RESET SECTION - BEGINS
;-------------------------------------------------------------------------
        LDP		#DP_EVA                 ;set data page
		SPLK   	#0, GPTCONA
		SPLK   	#0, T1CON
		SPLK   	#0, T2CON

		SPLK   	#0, COMCONA
		SPLK   	#0, ACTRA
		SPLK   	#0, DBTCONA
		SPLK   	#0, CAPCONA

;-------------------------------------------------------------------------
;		RESET SECTION - ENDS
;-------------------------------------------------------------------------

		LDP		#DP_EVA			;Set data page
		SPLK	#0h,GPTCONA		;GP timers are configured (no compare)
		SPLK   	#0080h, EVAIMRA	;Enable timer1 period interrupt
		SPLK	#7500,T1PR		;Period = 1 ms
		SPLK	#0h,T1CNT		;Initial value of the counter
		SPLK	#1244h,T1CON	;Input clock prescaler = 1/4
								;Disable timer compare operation
								;Enable timer operations
		CLRC   	INTM			;Enable maskable interrupts 
		
WAIT	B      	WAIT

;=====================================================================
; I S R  -  GISR2
;
; Description:	Calculates the number of seconds elapsed and 
;		accordingly outputs the numbers to the LEDs
;
; Modifies:	MSEC_CTR, SEC_CTR
;=====================================================================
GISR2	LDP		#MSEC_CTR			;Set data page
		LACC	MSEC_CTR
		ADD		#1
		SACL	MSEC_CTR			;Increment millisecond counter
		SUB		#1000				;Test if it reaches 1000
		BCND	BR1,NEQ				;If not, branch to BR1
		SPLK	#0, MSEC_CTR		;Else reset millisec counter
		LACC	SEC_CTR
		ADD		#1
		SACL	SEC_CTR				;Increment second counter
		OUT		SEC_CTR,LEDS		;Display second counter content
		SUB		#00Fh				;Test if it reaches its maximum
		BCND	BR1, NEQ			;If not, branch to BR1
		SPLK	#0, SEC_CTR			;Else reset second counter
BR1		LDP		#DP_EVA             ;Set data page
        SPLK    #0FFFFh, EVAIFRA	;clear all EVA group A interrupts
		CLRC	INTM				;Enable maskable interrupts
 		RET							;Return from interruption

;=====================================================================
; I S R  -  PHANTOM
;
; Description:	Dummy ISR, used to trap spurious interrupts.
;
; Modifies:	Nothing
;=====================================================================
PHANTOM 	B	PHANTOM
GISR1		RET
;GISR2		RET
GISR3		RET
GISR4		RET
GISR5		RET
GISR6		RET
